@@ -4,14 +4,17 @@ __all__ = [
     "TokenPattern",
     "SourceLocation",
     "explain_patterns",
+    "set_location",
+    "UNKNOWN_LOCATION",
     "InvalidSyntax",
     "UnexpectedEOF",
     "UnexpectedToken",
 ]
 
 
-__version__ = "0.5.0"
+__version__ = "1.0.0"
 
 from .error import *
+from .location import *
 from .stream import *
 from .token import *
