@@ -7,7 +7,8 @@ __all__ = [
 
 from typing import Sequence
 
-from .token import SourceLocation, Token, TokenPattern, explain_patterns
+from .location import SourceLocation
+from .token import Token, TokenPattern, explain_patterns
 
 
 class InvalidSyntax(Exception):
@@ -20,6 +21,12 @@ class InvalidSyntax(Exception):
     """
 
     location: SourceLocation
+    end_location: SourceLocation
+
+    def __init__(self, *args: object) -> None:
+        super().__init__(*args)
+        self.location = SourceLocation(0, 1, 1)
+        self.end_location = SourceLocation(0, 1, 1)
 
     def format(self, filename: str) -> str:
         """Return a string representing the error and its location in a given file.
