@@ -1,12 +1,16 @@
 __all__ = [
     "Token",
     "TokenPattern",
-    "SourceLocation",
     "explain_patterns",
 ]
 
 
-from typing import NamedTuple, Sequence, Tuple, Union
+from typing import NamedTuple, Sequence, Tuple, TypeVar, Union
+
+from .location import SourceLocation, set_location
+
+T = TypeVar("T")
+
 
 TokenPattern = Union[str, Tuple[str, str]]
 
@@ -17,28 +21,17 @@ def explain_patterns(patterns: Sequence[TokenPattern]) -> str:
         pattern if isinstance(pattern, str) else f"{pattern[0]} {pattern[1]!r}"
         for pattern in patterns
     ]
+    if not token_types:
+        return "nothing"
     if len(token_types) == 1:
         return token_types[0]
     *head, before_last, last = token_types
+    if len(head) > 6:
+        *head, before_last = head[:6]
+        last = f"{len(token_types) - 6} other tokens"
     return ", ".join(head + [f"{before_last} or {last}"])
 
 
-class SourceLocation(NamedTuple):
-    """Class representing a location within an input string."""
-
-    pos: int
-    lineno: int
-    colno: int
-
-    def format(self, filename: str, message: str) -> str:
-        """Return a message formatted with the given filename and the current location.
-
-        >>> SourceLocation(42, 3, 12).format("path/to/file.txt", "Some error message")
-        'path/to/file.txt:3:12: Some error message'
-        """
-        return f"{filename}:{self.lineno}:{self.colno}: {message}"
-
-
 class Token(NamedTuple):
     """Class representing a token."""
 
@@ -68,3 +61,24 @@ class Token(NamedTuple):
                 if self.type == pattern[0] and self.value == pattern[1]:
                     return True
         return False
+
+    def emit_error(self, exc: T) -> T:
+        """Add location information to invalid syntax exceptions.
+
+        This works exactly like :meth:`tokenstream.stream.TokenStream.emit_error` but it
+        associates the location of the token with the syntax error instead of the
+        head of the stream.
+
+        >>> stream = TokenStream("hello world")
+        >>> with stream.syntax(word=r"[a-z]+"):
+        ...     token = stream.expect()
+        ...     exc = token.emit_error(InvalidSyntax("goodbye"))
+        ...     raise exc
+        Traceback (most recent call last):
+        InvalidSyntax: goodbye
+        >>> exc.location
+        SourceLocation(pos=0, lineno=1, colno=1)
+        >>> exc.end_location
+        SourceLocation(pos=5, lineno=1, colno=6)
+        """
+        return set_location(exc, self)
